@@ -21,14 +21,8 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  *
- *  $Id$
- *
- *  A trivial dialog for editing comments
- *
  */
 
-#include <wx/wxprec.h>
-
 #include <wx/xrc/xmlres.h>
 #include <wx/config.h>
 #include <wx/textctrl.h>
@@ -56,7 +50,7 @@ BEGIN_EVENT_TABLE(CommentDialog, wxDialog)
    EVT_BUTTON(XRCID("clear"), CommentDialog::OnClear)
 END_EVENT_TABLE()
 
-void CommentDialog::OnClear(wxCommandEvent& event)
+void CommentDialog::OnClear(wxCommandEvent&)
 {
     m_text->Clear();
 }
