@@ -21,17 +21,13 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE. 
  *
- *  $Id$
- *
- *  A trivial dialog for editing comments
- *
  */
 
 #ifndef _COMMENTDLG_H_
 #define _COMMENTDLG_H_
 
 #include <wx/dialog.h>
-class WXDLLEXPORT wxTextCtrl;
+class WXDLLIMPEXP_FWD_CORE wxTextCtrl;
 
 /** CommentDialog is a very simple dialog that lets the user edit
     catalog comments. Comment consists of one or more lines that
