@@ -21,15 +21,9 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  *
- *  $Id$
- *
- *  Database of available parsers
- *
  */
 
 
-#include <wx/wxprec.h>
-
 #include <wx/wfstream.h>
 #include <wx/config.h>
 #include <wx/tokenzr.h>
@@ -113,7 +107,7 @@ int ParsersDB::FindParser(const wxString& name)
 
 wxArrayString Parser::SelectParsable(const wxArrayString& files)
 {
-    wxStringTokenizer tkn(Extensions, _T(";"));
+    wxStringTokenizer tkn(Extensions, _T(";, \t"), wxTOKEN_STRTOK);
     wxString wildcard;
     wxArrayString result;
     size_t i;
