@@ -21,10 +21,6 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  *
- *  $Id$
- *
- *  Database of available parsers
- *
  */
 
 #ifndef _PARSER_H_
@@ -34,6 +30,8 @@
 #include <wx/dynarray.h>
 #include <wx/string.h>
 
+class WXDLLIMPEXP_FWD_BASE wxConfigBase;
+
 
 /** This class holds information about an external parser. It does
     \b not do any parsing. The only functionality it provides
@@ -85,8 +83,6 @@ class Parser
 
 WX_DECLARE_OBJARRAY(Parser, ParserArray);
 
-class WXDLLEXPORT wxConfigBase;
-
 /** Database of all available parsers. This class is regular pseudo-template
     dynamic wxArray with additional methods for storing its content to
     wxConfig object and retrieving it.
