@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://www.poedit.net)
  *
- *  Copyright (C) 2000-2005 Vaclav Slavik
+ *  Copyright (C) 2000-2012 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -21,13 +21,8 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  *
- *  $Id$
- *
- *  Preferences settings dialog
- *
  */
 
-#include <wx/wxprec.h>
 #include "wxeditablelistbox.h"
 #include <wx/textctrl.h>
 #include <wx/button.h>
@@ -36,8 +31,8 @@
 #include <wx/choicdlg.h>
 #include <wx/spinctrl.h>
 #include <wx/notebook.h>
-#include <wx/fontdlg.h>
 #include <wx/fontutil.h>
+#include <wx/fontpicker.h>
 #include <wx/xrc/xmlres.h>
 
 #include "prefsdlg.h"
@@ -47,8 +42,12 @@
 #include "transmemupd_wizard.h"
 #include "chooselang.h"
 
+#ifdef __WXMSW__
+#include <winsparkle.h>
+#endif
+
 #ifdef USE_SPARKLE
-#include "osx/userdefaults.h"
+#include "osx_helpers.h"
 #endif // USE_SPARKLE
 
 PreferencesDialog::PreferencesDialog(wxWindow *parent)
@@ -107,7 +106,7 @@ void PreferencesDialog::TransferTo(wxConfigBase *cfg)
     XRCCTRL(*this, "user_email", wxTextCtrl)->SetValue(
                 cfg->Read(_T("translator_email"), wxEmptyString));
     XRCCTRL(*this, "compile_mo", wxCheckBox)->SetValue(
-                cfg->Read(_T("compile_mo"), true));
+                cfg->Read(_T("compile_mo"), (long)true));
     XRCCTRL(*this, "show_summary", wxCheckBox)->SetValue(
                 cfg->Read(_T("show_summary"), true));
     XRCCTRL(*this, "manager_startup", wxCheckBox)->SetValue(
@@ -116,32 +115,26 @@ void PreferencesDialog::TransferTo(wxConfigBase *cfg)
                 (bool)cfg->Read(_T("focus_to_text"), (long)false));
     XRCCTRL(*this, "comment_window_editable", wxCheckBox)->SetValue(
                 (bool)cfg->Read(_T("comment_window_editable"), (long)false));
-    XRCCTRL(*this, "ext_editor", wxComboBox)->SetValue(
-                cfg->Read(_T("ext_editor"), wxEmptyString));
-    XRCCTRL(*this, "open_editor_immediately", wxCheckBox)->SetValue(
-                cfg->Read(_T("open_editor_immediately"), (long)false));
     XRCCTRL(*this, "keep_crlf", wxCheckBox)->SetValue(
                 (bool)cfg->Read(_T("keep_crlf"), true));
 #ifdef USE_SPELLCHECKING
     XRCCTRL(*this, "enable_spellchecking", wxCheckBox)->SetValue(
                 (bool)cfg->Read(_T("enable_spellchecking"), true));
 #endif
-    
+
     XRCCTRL(*this, "use_font_list", wxCheckBox)->SetValue(
                 (bool)cfg->Read(_T("custom_font_list_use"), (long)false));
     XRCCTRL(*this, "use_font_text", wxCheckBox)->SetValue(
                 (bool)cfg->Read(_T("custom_font_text_use"), (long)false));
-    XRCCTRL(*this, "fontname_list", wxTextCtrl)->SetValue(
-            cfg->Read(_T("custom_font_list_name"), wxEmptyString));
-    XRCCTRL(*this, "fontname_text", wxTextCtrl)->SetValue(
-            cfg->Read(_T("custom_font_text_name"), wxEmptyString));
+    XRCCTRL(*this, "font_list", wxFontPickerCtrl)->SetSelectedFont(
+            wxFont(cfg->Read(_T("custom_font_list_name"), wxEmptyString)));
+    XRCCTRL(*this, "font_text", wxFontPickerCtrl)->SetSelectedFont(
+            wxFont(cfg->Read(_T("custom_font_text_name"), wxEmptyString)));
 
     wxString format = cfg->Read(_T("crlf_format"), _T("unix"));
     int sel;
     if (format == _T("win")) sel = 1;
-    else if (format == _T("mac")) sel = 2;
-    else if (format == _T("native")) sel = 3;
-    else /* _T("unix") */ sel = 0;
+    else /* _T("unix") or obsolete settings */ sel = 0;
 
     XRCCTRL(*this, "crlf_format", wxChoice)->SetSelection(sel);
 
@@ -160,9 +153,6 @@ void PreferencesDialog::TransferTo(wxConfigBase *cfg)
         list->SetSelection(0);
 
 #ifdef USE_TRANSMEM        
-    XRCCTRL(*this, "tm_dbpath", wxTextCtrl)->SetValue(
-                cfg->Read(_T("TM/database_path"), wxEmptyString));
-
     wxStringTokenizer tkn(cfg->Read(_T("TM/languages"), wxEmptyString), _T(":"));
     wxArrayString langs;
     while (tkn.HasMoreTokens()) langs.Add(tkn.GetNextToken());
@@ -178,8 +168,12 @@ void PreferencesDialog::TransferTo(wxConfigBase *cfg)
 
 #ifdef USE_SPARKLE
     XRCCTRL(*this, "auto_updates", wxCheckBox)->SetValue(
-                UserDefaults::GetBoolValue("SUEnableAutomaticChecks"));
+                (bool)UserDefaults_GetBoolValue("SUEnableAutomaticChecks"));
 #endif // USE_SPARKLE
+#ifdef __WXMSW__
+    XRCCTRL(*this, "auto_updates", wxCheckBox)->SetValue(
+                (bool)win_sparkle_get_automatic_check_for_updates());
+#endif
 }
  
             
@@ -199,28 +193,26 @@ void PreferencesDialog::TransferFrom(wxConfigBase *cfg)
                 XRCCTRL(*this, "focus_to_text", wxCheckBox)->GetValue());
     cfg->Write(_T("comment_window_editable"), 
                 XRCCTRL(*this, "comment_window_editable", wxCheckBox)->GetValue());
-    cfg->Write(_T("ext_editor"), 
-                XRCCTRL(*this, "ext_editor", wxComboBox)->GetValue());
-    cfg->Write(_T("open_editor_immediately"), 
-                XRCCTRL(*this, "open_editor_immediately", wxCheckBox)->GetValue());
     cfg->Write(_T("keep_crlf"), 
                 XRCCTRL(*this, "keep_crlf", wxCheckBox)->GetValue());
 #ifdef USE_SPELLCHECKING
     cfg->Write(_T("enable_spellchecking"), 
                 XRCCTRL(*this, "enable_spellchecking", wxCheckBox)->GetValue());
 #endif
-    
+   
+    wxFont listFont = XRCCTRL(*this, "font_list", wxFontPickerCtrl)->GetSelectedFont();
+    wxFont textFont = XRCCTRL(*this, "font_text", wxFontPickerCtrl)->GetSelectedFont();
+
     cfg->Write(_T("custom_font_list_use"),
-                XRCCTRL(*this, "use_font_list", wxCheckBox)->GetValue());
+               listFont.IsOk() && XRCCTRL(*this, "use_font_list", wxCheckBox)->GetValue());
     cfg->Write(_T("custom_font_text_use"),
-                XRCCTRL(*this, "use_font_text", wxCheckBox)->GetValue());
-    cfg->Write(_T("custom_font_list_name"),
-                XRCCTRL(*this, "fontname_list", wxTextCtrl)->GetValue());
-    cfg->Write(_T("custom_font_text_name"),
-                XRCCTRL(*this, "fontname_text", wxTextCtrl)->GetValue());
- 
-    static const wxChar *formats[] = 
-        { _T("unix"), _T("win"), _T("mac"), _T("native") };
+               textFont.IsOk() && XRCCTRL(*this, "use_font_text", wxCheckBox)->GetValue());
+    if ( listFont.IsOk() )
+        cfg->Write(_T("custom_font_list_name"), listFont.GetNativeFontInfoDesc());
+    if ( textFont.IsOk() )
+        cfg->Write(_T("custom_font_text_name"), textFont.GetNativeFontInfoDesc());
+
+    static const wxChar *formats[] = { _T("unix"), _T("win") };
     cfg->Write(_T("crlf_format"), formats[
                 XRCCTRL(*this, "crlf_format", wxChoice)->GetSelection()]);
                
@@ -236,8 +228,6 @@ void PreferencesDialog::TransferFrom(wxConfigBase *cfg)
         languages << langs[i];
     }
     cfg->Write(_T("TM/languages"), languages);
-    cfg->Write(_T("TM/database_path"),
-                XRCCTRL(*this, "tm_dbpath", wxTextCtrl)->GetValue());
     cfg->Write(_T("TM/max_omitted"), 
                 (long)XRCCTRL(*this, "tm_omits", wxSpinCtrl)->GetValue());
     cfg->Write(_T("TM/max_delta"), 
@@ -247,9 +237,13 @@ void PreferencesDialog::TransferFrom(wxConfigBase *cfg)
 #endif
 
 #ifdef USE_SPARKLE
-    UserDefaults::SetBoolValue("SUEnableAutomaticChecks",
+    UserDefaults_SetBoolValue("SUEnableAutomaticChecks",
                 XRCCTRL(*this, "auto_updates", wxCheckBox)->GetValue());
 #endif // USE_SPARKLE
+#ifdef __WXMSW__
+    win_sparkle_set_automatic_check_for_updates(
+                XRCCTRL(*this, "auto_updates", wxCheckBox)->GetValue());
+#endif
 }
 
 
@@ -258,25 +252,36 @@ BEGIN_EVENT_TABLE(PreferencesDialog, wxDialog)
    EVT_BUTTON(XRCID("parser_new"), PreferencesDialog::OnNewParser)
    EVT_BUTTON(XRCID("parser_edit"), PreferencesDialog::OnEditParser)
    EVT_BUTTON(XRCID("parser_delete"), PreferencesDialog::OnDeleteParser)
-   EVT_BUTTON(XRCID("fontpicker_list"), PreferencesDialog::OnChooseListFont)
-   EVT_BUTTON(XRCID("fontpicker_text"), PreferencesDialog::OnChooseTextFont)
 #ifdef USE_TRANSMEM
    EVT_BUTTON(XRCID("tm_addlang"), PreferencesDialog::OnTMAddLang)
-   EVT_BUTTON(XRCID("tm_browsedbpath"), PreferencesDialog::OnTMBrowseDbPath)
    EVT_BUTTON(XRCID("tm_generate"), PreferencesDialog::OnTMGenerate)
 #endif
 #if NEED_CHOOSELANG_UI
    EVT_BUTTON(XRCID("ui_language"), PreferencesDialog::OnUILanguage)
 #endif
+   EVT_UPDATE_UI(XRCID("font_list"), PreferencesDialog::OnUpdateUIFontList)
+   EVT_UPDATE_UI(XRCID("font_text"), PreferencesDialog::OnUpdateUIFontText)
 END_EVENT_TABLE()
 
 #if NEED_CHOOSELANG_UI
-void PreferencesDialog::OnUILanguage(wxCommandEvent& event)
+void PreferencesDialog::OnUILanguage(wxCommandEvent&)
 {
     ChangeUILanguage();
 }
 #endif
 
+
+void PreferencesDialog::OnUpdateUIFontList(wxUpdateUIEvent& event)
+{
+    event.Enable(XRCCTRL(*this, "use_font_list", wxCheckBox)->GetValue());
+}
+
+void PreferencesDialog::OnUpdateUIFontText(wxUpdateUIEvent& event)
+{
+    event.Enable(XRCCTRL(*this, "use_font_text", wxCheckBox)->GetValue());
+}
+
+
 bool PreferencesDialog::EditParser(int num)
 {
     wxDialog dlg;
@@ -307,7 +312,7 @@ bool PreferencesDialog::EditParser(int num)
         return false;
 }
 
-void PreferencesDialog::OnNewParser(wxCommandEvent& event)
+void PreferencesDialog::OnNewParser(wxCommandEvent&)
 {
     Parser info;
     m_parsers.Add(info);
@@ -325,12 +330,12 @@ void PreferencesDialog::OnNewParser(wxCommandEvent& event)
     }
 }
 
-void PreferencesDialog::OnEditParser(wxCommandEvent& event)
+void PreferencesDialog::OnEditParser(wxCommandEvent&)
 {
     EditParser(XRCCTRL(*this, "parsers_list", wxListBox)->GetSelection());
 }
 
-void PreferencesDialog::OnDeleteParser(wxCommandEvent& event)
+void PreferencesDialog::OnDeleteParser(wxCommandEvent&)
 {
     size_t index = XRCCTRL(*this, "parsers_list", wxListBox)->GetSelection();
     m_parsers.RemoveAt(index);
@@ -344,7 +349,7 @@ void PreferencesDialog::OnDeleteParser(wxCommandEvent& event)
 
 #ifdef USE_TRANSMEM
 
-void PreferencesDialog::OnTMAddLang(wxCommandEvent& event)
+void PreferencesDialog::OnTMAddLang(wxCommandEvent&)
 {
     wxArrayString lngs;
     int index;
@@ -363,61 +368,12 @@ void PreferencesDialog::OnTMAddLang(wxCommandEvent& event)
     }
 }
 
-void PreferencesDialog::OnTMBrowseDbPath(wxCommandEvent& event)
+void PreferencesDialog::OnTMGenerate(wxCommandEvent&)
 {
-    wxDirDialog dlg(this, _("Select directory"), 
-                    XRCCTRL(*this, "tm_dbpath", wxTextCtrl)->GetValue());
-    if (dlg.ShowModal() == wxID_OK)
-        XRCCTRL(*this, "tm_dbpath", wxTextCtrl)->SetValue(dlg.GetPath());
-}
-
-
-void PreferencesDialog::OnTMGenerate(wxCommandEvent& event)
-{
-    wxString dbPath = XRCCTRL(*this, "tm_dbpath", wxTextCtrl)->GetValue();
-        // VS: we can't get it from TM/database_path key in wxConfig object
-        //     because it wasn't update yet with information from the dialog
-        //     (which happens when the users presses OK) but we still won't
-        //     to use the path entered by the user...
-
     wxArrayString langs;
     XRCCTRL(*this, "tm_langs", wxEditableListBox)->GetStrings(langs);
 
-    RunTMUpdateWizard(this, dbPath, langs);
+    RunTMUpdateWizard(this, langs);
 }
 
-#endif // USE_TRANSMEM
-        
-void PreferencesDialog::DoChooseFont(wxTextCtrl *nameField)
-{
-    wxFont font;
-    wxString fontname = nameField->GetValue();
-    if (!fontname.empty())
-    {
-        wxNativeFontInfo fi;
-        fi.FromString(fontname);
-        font.SetNativeFontInfo(fi);
-    }
-    wxFontData data;
-    data.EnableEffects(false);
-    data.SetAllowSymbols(false);
-    data.SetInitialFont(font);
-    wxFontDialog dlg(this, data);
-
-    if (dlg.ShowModal() == wxID_OK)
-    {
-        data = dlg.GetFontData();
-        font = data.GetChosenFont();
-        nameField->SetValue(font.GetNativeFontInfoDesc());
-    }
-}
-
-void PreferencesDialog::OnChooseListFont(wxCommandEvent& event)
-{
-    DoChooseFont(XRCCTRL(*this, "fontname_list", wxTextCtrl));
-}
-
-void PreferencesDialog::OnChooseTextFont(wxCommandEvent& event)
-{
-    DoChooseFont(XRCCTRL(*this, "fontname_text", wxTextCtrl));
-}
+#endif // USE_TRANSMEM
\ No newline at end of file
