@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://www.poedit.net)
  *
- *  Copyright (C) 2000-2005 Vaclav Slavik
+ *  Copyright (C) 2000-2012 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -21,10 +21,6 @@
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  *
- *  $Id$
- *
- *  Preferences settings dialog
- *
  */
 
 #ifndef _PREFSDLG_H_
@@ -35,7 +31,7 @@
 #include "parser.h"
 #include "chooselang.h"
 
-class WXDLLEXPORT wxConfigBase;
+class WXDLLIMPEXP_FWD_BASE wxConfigBase;
 
 /** Preferences dialog for setting user's identity, parsers and other
     global, catalog-independent settings.
@@ -60,7 +56,6 @@ class PreferencesDialog : public wxDialog
 
 #ifdef USE_TRANSMEM
         void OnTMAddLang(wxCommandEvent& event);
-        void OnTMBrowseDbPath(wxCommandEvent& event);
         void OnTMGenerate(wxCommandEvent& event);
 #endif
 
@@ -70,12 +65,11 @@ class PreferencesDialog : public wxDialog
         void OnNewParser(wxCommandEvent& event);
         void OnEditParser(wxCommandEvent& event);
         void OnDeleteParser(wxCommandEvent& event);
+        void OnUpdateUIFontList(wxUpdateUIEvent& event);
+        void OnUpdateUIFontText(wxUpdateUIEvent& event);
+
         /// Called to launch dialog for editting parser properties.
         bool EditParser(int num);
-
-        void OnChooseListFont(wxCommandEvent& event);
-        void OnChooseTextFont(wxCommandEvent& event);
-        void DoChooseFont(wxTextCtrl *nameField);
 };
 
 
